var __defProp = Object.defineProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};

// src/Adaptor.js
var Adaptor_exports = {};
__export(Adaptor_exports, {
  alterState: () => alterState,
  arrayToString: () => arrayToString,
  create: () => create,
  cursor: () => cursor,
  dataPath: () => dataPath,
  dataValue: () => dataValue,
  dateFns: () => dateFns,
  destroy: () => destroy,
  each: () => each,
  execute: () => execute,
  field: () => field,
  fields: () => fields,
  fn: () => fn,
  fnIf: () => fnIf,
  get: () => get,
  lastReferenceValue: () => lastReferenceValue,
  merge: () => merge,
  sourceValue: () => sourceValue,
  update: () => update,
  upsert: () => upsert,
  util: () => util2
});
import {
  execute as commonExecute,
  composeNextState as composeNextState2
} from "@openfn/language-common";
import { expandReferences } from "@openfn/language-common/util";

// src/Utils.js
import { composeNextState, util } from "@openfn/language-common";
import {
  request as commonRequest,
  makeBasicAuthHeader
} from "@openfn/language-common/util";
var prepareNextState = (state, response) => {
  const { body, ...responseWithoutBody } = response;
  return {
    ...composeNextState(state, body),
    response: responseWithoutBody
  };
};
async function fetchAndLog(method, url, opts = {}) {
  var _a, _b, _c, _d, _e;
  try {
    const response = await commonRequest(method, url, opts);
    const { statusCode, duration } = response;
    const urlWithQuery = Object.keys(opts.query || {}).length ? `${url}?${new URLSearchParams(opts.query).toString()}` : url;
    const message = `${method} ${urlWithQuery} - ${statusCode} in ${duration}ms`;
    if (response instanceof Error) {
      console.error(message);
    } else {
      console.log(message);
    }
    response.url = urlWithQuery;
    (_a = opts._onrequest) == null ? void 0 : _a.call(opts, {
      url: urlWithQuery,
      query: opts.query,
      method
    });
    return response;
  } catch (e) {
    if (((_c = (_b = e.body) == null ? void 0 : _b.error) == null ? void 0 : _c.detail) && ((_e = (_d = e.body) == null ? void 0 : _d.error) == null ? void 0 : _e.message)) {
      delete e.body.error.detail;
    }
    throw e;
  }
}
function validateCredentials(state) {
  const config = state.configuration || {};
  for (const key of ["instanceUrl"]) {
    if (!config[key]) {
      util.throwError("INVALID_CREDENTIALS", {
        description: `configuration.${key}`,
        fix: "Ensure that all required credentials are set on state.configuration"
      });
    }
  }
}
async function requestWithPagination(state, path, options = {}) {
  var _a;
  const results = [];
  let { pageSize, startIndex, limit, max } = options;
  const isUsingDefaultMax = max === void 0;
  const maxResults = max ?? limit ?? 1e4;
  let requestOptions = { ...options };
  let shouldFetchMoreContent = false;
  let isFirstRequest = true;
  const didUserPassLimit = Boolean(max || limit);
  do {
    requestOptions.query ?? (requestOptions.query = {});
    if (!isNaN(startIndex)) {
      requestOptions.query.startIndex = startIndex;
    }
    if (limit) {
      requestOptions.query.limit = limit;
    } else if (didUserPassLimit || !isFirstRequest) {
      requestOptions.query.limit = Math.min(
        pageSize || maxResults,
        maxResults - results.length
      );
    } else if (!isNaN(pageSize)) {
      requestOptions.query.limit = pageSize;
    }
    const response = await request(state, "GET", path, requestOptions);
    if (response.body.results) {
      results.push(...response.body.results);
      if (!startIndex) {
        startIndex = 1;
      }
      startIndex += response.body.results.length;
    } else {
      results.push(response.body);
    }
    if (isFirstRequest) {
      if (!pageSize) {
        pageSize = results.length;
      }
    }
    isFirstRequest = false;
    const hasMoreContent = (_a = response.body.links) == null ? void 0 : _a.find(
      (link) => (link == null ? void 0 : link.rel) === "next"
    );
    if (isUsingDefaultMax && hasMoreContent && results.length === maxResults) {
      console.warn(
        `Warning: The default maximum number of items has been reached (${maxResults}), but more items are available on the server. To download all available items, make another request with startIndex=${maxResults + 1} or set max to Infinity`
      );
    }
    shouldFetchMoreContent = !limit && results.length < maxResults && hasMoreContent;
  } while (shouldFetchMoreContent);
  return results;
}
async function request(state, method, path, options = {}) {
  validateCredentials(state);
  const { username, password, instanceUrl: baseUrl } = state.configuration;
  const {
    query = {},
    data = {},
    headers = { "content-type": "application/json" },
    parseAs = "json",
    _onrequest,
    errors
  } = options;
  if (baseUrl.length <= 0) {
    throw new Error(
      "Invalid instanceUrl. Include instanceUrl in state.configuration"
    );
  }
  const isAbsoluteUrl = /^(https?:|\/\/)/i.test(path);
  if (isAbsoluteUrl) {
    throw new Error(
      `Invalid path argument: "${path}" appears to be an absolute URL. Please provide a relative path`
    );
  }
  const authHeaders = makeBasicAuthHeader(username, password);
  const requestOptions = {
    body: data,
    headers: {
      ...authHeaders,
      ...headers
    },
    query,
    parseAs,
    baseUrl,
    errors,
    _onrequest
  };
  return fetchAndLog(method, path, requestOptions);
}
function cleanPath(path) {
  return path.replace(/([^:]\/)\/+/g, "$1").replace(/\/$/, "");
}

// src/Adaptor.js
import {
  alterState,
  arrayToString,
  cursor,
  dataPath,
  dataValue,
  dateFns,
  each,
  field,
  fields,
  fn,
  fnIf,
  lastReferenceValue,
  merge,
  sourceValue,
  util as util2
} from "@openfn/language-common";
function execute(...operations) {
  const initialState = {
    references: [],
    data: null
  };
  return (state) => {
    return commonExecute(...operations)({ ...initialState, ...state });
  };
}
function get(path, options = {}) {
  return async (state) => {
    var _a;
    const [resolvedPath, resolvedOptions] = expandReferences(
      state,
      path,
      options
    );
    if (resolvedOptions.limit) {
      const keysToRemove = Object.keys(resolvedOptions).filter(
        (k) => k.match(/^(max|pageSize)$/)
      );
      if (keysToRemove.length) {
        console.warn(
          `Warning: ignoring option [${keysToRemove.join(
            ","
          )}] as "limit" is set`
        );
        delete resolvedOptions.max;
        delete resolvedOptions.pageSize;
      }
    }
    let { max, singleton, limit, pageSize, query, ...queryParams } = resolvedOptions;
    if (singleton) {
      max = 1;
    }
    if (resolvedOptions.query) {
      queryParams.q = resolvedOptions.query;
    }
    const requestOptions = {
      baseUrl: (_a = state.configuration) == null ? void 0 : _a.instanceUrl,
      query: queryParams,
      max,
      limit,
      pageSize
    };
    try {
      let result = await requestWithPagination(
        state,
        cleanPath(`/ws/rest/v1/${resolvedPath}`),
        requestOptions
      );
      if (singleton) {
        result = result[0];
      } else {
        console.log(`get() downloaded ${result.length} resources`);
      }
      return composeNextState2(state, result);
    } catch (e) {
      if (e.statusCode === 400 && !queryParams.q) {
        e.fix = "Many OpenMRS list endpoints expect a query to be passed - try setting the query option on get()";
      }
      throw e;
    }
  };
}
function create(path, data) {
  return async (state) => {
    const [resolvedPath, resolvedData] = expandReferences(state, path, data);
    console.log(`Preparing to create ${resolvedPath}`);
    const { instanceUrl: baseUrl } = state.configuration;
    const response = await request(
      state,
      "POST",
      cleanPath(`/ws/rest/v1/${resolvedPath}`),
      {
        baseUrl,
        data: resolvedData
      }
    );
    console.log(`Successfully created ${resolvedPath}`);
    return composeNextState2(state, response.body);
  };
}
function update(path, data) {
  return async (state) => {
    const [resolvedPath, resolvedData] = expandReferences(state, path, data);
    console.log(`Preparing to update ${resolvedPath}`);
    const { instanceUrl: baseUrl } = state.configuration;
    const response = await request(
      state,
      "POST",
      cleanPath(`/ws/rest/v1/${resolvedPath}`),
      {
        baseUrl,
        data: resolvedData
      }
    );
    console.log(`Successfully updated ${resolvedPath}`);
    return composeNextState2(state, response.body);
  };
}
function upsert(path, data, params = {}) {
  return async (state) => {
    var _a;
    const [resolvedPath, resolvedData, resolvedParams] = expandReferences(
      state,
      path,
      data,
      params
    );
    console.log(`Preparing composed upsert on ${resolvedPath}`);
    const { instanceUrl: baseUrl } = state.configuration;
    const res = await request(
      state,
      "GET",
      cleanPath(`/ws/rest/v1/${resolvedPath}`),
      {
        baseUrl,
        query: resolvedParams,
        errors: { 404: false }
      }
    );
    let found = res.statusCode <= 400;
    const count = ((_a = res.body.results) == null ? void 0 : _a.length) ?? 1;
    if (found && count > 1) {
      throw new RangeError(
        `Found more than one record for your request; cannot upsert on non-unique attribute.`
      );
    } else if (!found || count === 0) {
      console.log(`${resolvedPath} not found: creating new resource`);
      const path2 = resolvedParams.q ? resolvedPath : resolvedPath.split("/").slice(0, -1).join("/");
      return create(path2, resolvedData)(state);
    } else {
      const path2 = resolvedParams.q ? `${resolvedPath}/${res.body.results[0].uuid}` : resolvedPath;
      console.log(`${path2} found: updating existing resource`);
      return update(path2, resolvedData)(state);
    }
  };
}
function destroy(path, options) {
  return async (state) => {
    const [resolvedPath, resolvedOptions = {}] = expandReferences(
      state,
      path,
      options
    );
    const { instanceUrl: baseUrl } = state.configuration;
    const response = await request(
      state,
      "DELETE",
      cleanPath(`/ws/rest/v1/${resolvedPath}`),
      {
        baseUrl,
        query: resolvedOptions
      }
    );
    return composeNextState2(state, response.body);
  };
}

// src/fhir.js
var fhir_exports = {};
__export(fhir_exports, {
  get: () => get2
});
import { expandReferences as expandReferences2 } from "@openfn/language-common/util";

// src/fhir.utils.js
import {
  request as commonRequest2,
  makeBasicAuthHeader as makeBasicAuthHeader2,
  logResponse
} from "@openfn/language-common/util";
async function request2(state, method, path, data, params) {
  var _a, _b, _c;
  const {
    instanceUrl,
    username,
    password,
    fhirVersion = "R4"
  } = state.configuration;
  const headers = makeBasicAuthHeader2(username, password);
  const queryParams = mapParams(params);
  const options = {
    body: data,
    headers: {
      ...headers,
      "content-type": "application/json"
    },
    query: queryParams,
    parseAs: "json"
  };
  const url = `${instanceUrl}/ws/fhir2/${fhirVersion}/${path}`;
  let allResponses;
  let query = options == null ? void 0 : options.query;
  let allowPagination = isNaN(query == null ? void 0 : query._getpagesoffset);
  do {
    const requestOptions = query ? { ...options, query } : options;
    const response = await commonRequest2(method, url, requestOptions);
    logResponse(response);
    if (allResponses) {
      allResponses.body.entry.push(...response.body.entry);
    } else {
      allResponses = response;
    }
    const nextUrl = (_c = (_b = (_a = response == null ? void 0 : response.body) == null ? void 0 : _a.link) == null ? void 0 : _b.find((l) => l.relation === "next")) == null ? void 0 : _c.url;
    if (nextUrl) {
      console.log(`Fetched ${response.body.entry.length} results`);
      console.log(`Fetching next page from ${nextUrl}`);
      const urlObj = new URL(nextUrl);
      const params2 = new URLSearchParams(urlObj.search);
      const paramsObject = Object.fromEntries(params2.entries());
      query = { ...query, ...paramsObject };
    } else {
      delete allResponses.body.link;
      break;
    }
  } while (allowPagination);
  return allResponses;
}
var paramsForAllResources = {
  id: "_id",
  lastUpdated: "_lastUpdated",
  tag: "_tag",
  profile: "_profile",
  security: "_security",
  text: "_text",
  content: "_content",
  list: "_list",
  has: "_has",
  type: "_type"
};
var searchParams = {
  sort: "_sort",
  count: "_count",
  include: "_include",
  revinclude: "_revinclude",
  summary: "_summary",
  total: "_total",
  elements: "_elements",
  contained: "_contained",
  containedType: "_containedType"
};
var openMRSPagingParams = {
  getPagesOffset: "_getpagesoffset",
  getPages: "_getpages",
  budleType: "_bundletype"
};
function mapParams(passedParams) {
  const mappedParams = {};
  const allParams = {
    ...paramsForAllResources,
    ...searchParams,
    ...openMRSPagingParams
  };
  for (const key in passedParams) {
    if (allParams[key]) {
      mappedParams[allParams[key]] = passedParams[key];
    } else {
      mappedParams[key] = passedParams[key];
    }
  }
  return mappedParams;
}

// src/fhir.js
function get2(path, query, callback = (s) => s) {
  return async (state) => {
    const [resolvedPath, resolvedQuery = {}] = expandReferences2(
      state,
      path,
      query
    );
    const response = await request2(
      state,
      "GET",
      resolvedPath,
      {},
      resolvedQuery
    );
    return prepareNextState(state, response, callback);
  };
}

// src/http.js
var http_exports = {};
__export(http_exports, {
  delete: () => _delete,
  get: () => get3,
  post: () => post,
  request: () => request3
});
import { expandReferences as expandReferences3 } from "@openfn/language-common/util";
function request3(method, path, options = {}) {
  return async (state) => {
    const [resolvedMethod, resolvedPath, resolvedOptions = {}] = expandReferences3(state, method, path, options);
    const response = await request(
      state,
      resolvedMethod,
      resolvedPath,
      resolvedOptions
    );
    return prepareNextState(state, response);
  };
}
function get3(path, options = {}) {
  return async (state) => {
    const [resolvedPath, resolvedOptions] = expandReferences3(
      state,
      path,
      options
    );
    const response = await request(
      state,
      "GET",
      resolvedPath,
      resolvedOptions
    );
    return prepareNextState(state, response);
  };
}
function post(path, data, options = {}) {
  return async (state) => {
    const [resolvedPath, resolvedData, resolvedOptions] = expandReferences3(
      state,
      path,
      data,
      options
    );
    const optionsObject = {
      data: resolvedData,
      ...resolvedOptions
    };
    const response = await request(
      state,
      "POST",
      resolvedPath,
      optionsObject
    );
    return prepareNextState(state, response);
  };
}
function _delete(path, options = {}) {
  return async (state) => {
    const [resolvedPath, resolvedOptions] = expandReferences3(
      state,
      path,
      options
    );
    const response = await request(
      state,
      "DELETE",
      resolvedPath,
      resolvedOptions
    );
    return prepareNextState(state, response);
  };
}

// src/index.js
var src_default = Adaptor_exports;
export {
  alterState,
  arrayToString,
  create,
  cursor,
  dataPath,
  dataValue,
  dateFns,
  src_default as default,
  destroy,
  each,
  execute,
  fhir_exports as fhir,
  field,
  fields,
  fn,
  fnIf,
  get,
  http_exports as http,
  lastReferenceValue,
  merge,
  sourceValue,
  update,
  upsert,
  util2 as util
};
